import os

"""
    Update files via HTTPS only.

    Makes sure all needed libs are installed
    to make https calls.
"""
# Declare all files to be updated

updatefiles = ["AutoUpdate.py",
               "path2poserobj.py",
               "PoserObject2Json.py",
               "PoserTalkr.py",
               "PoserTalkrClientTest.py",
               "tdir.py"
               ]

UPDATED = False  # True if update(...) did anything.
HOST = "adp.spdns.org"  # Website we access for updates.


def update(to_path=None, force=False):
    """
    Function to call to check for updates.
    "to_path" is the directory where updates
    files should be written to.
    Any filename from "updatefile" is checked
    and updated if needed.
    If a file exists already, the size of this file
    is compared to the filesize returned from
    the web-request.
    If "force" is True, missing files are created.
    ----------------------------------------------
    This is a dummy function, overwritten if
    some imports could be done.
    """
    pass


__all__ = ["HOST", "UPDATED", "update"]

try:
    import poser
except ImportError:
    poser = None
    version = "---"
else:
    version = poser.Version()

currentdir = os.path.abspath(os.curdir)

try:
    import httplib
except ImportError:
    pass  # no update available
else:
    try:
        import ssl
    except ImportError:
        pass  # no update available
    else:
        def setLibDir(path):
            """
            Change current directory to path
            or create path in current directory if it not exist.
            :param path:
            :return:
            """
            global currentdir
            if not currentdir.endswith(path):
                if not os.path.exists(path):
                    old = currentdir
                    currentdir = os.path.join(currentdir, path)
                    try:
                        os.mkdir(currentdir)
                    except Exception:
                        currentdir = old
                        return False
                os.chdir(currentdir)
            return True


        def update(to_path="PoserTalkr", force=False):
            """
            Redeclare function update
            """
            global UPDATED

            setLibDir(to_path)

            try:
                ssl.create_default_context()
            except Exception:
                # Not really an error, but maybe a security risk.
                # Https may work anyway.
                pass

            make_requests = []
            r = con = None
            headers = {
                "User-Agent": "PoserTalkr_%s" % version
            }

            while updatefiles:
                reqfile = updatefiles.pop()
                if not os.path.isfile(reqfile):
                    # Not available in the current directory.
                    # Lets check if we need to download anyway.
                    if not force:
                        continue  # while updatefiles

                # Lets check if we are able to update the file.
                if not os.access(reqfile, os.W_OK):
                    # force or not, we are not allowed to write
                    # to this file.
                    continue

                con = httplib.HTTPSConnection(HOST, timeout=2)
                if con:
                    con.request("HEAD", "/%s" % reqfile, headers=headers)
                    r = con.getresponse()
                    if r and r.status < 400:
                        h = r.getheaders()
                        size = r.getheader("content-length")
                        if int(size) == 0:
                            # better no update if size is zero.
                            continue
                        fsize = os.stat(reqfile).st_size
                        if force or int(size) != fsize:
                            # seems to be a different file.
                            make_requests.append(reqfile)
                else:
                    break  # while updatefiles

            while make_requests:
                reqfile = make_requests.pop()
                con = httplib.HTTPSConnection(HOST, timeout=2)
                if con:
                    con.request("GET", "/%s" % reqfile, headers=headers)
                    r = con.getresponse()
                    if r.status == 200:
                        if os.path.exists(reqfile + "c"):
                            os.remove(reqfile + "c")
                        with open(reqfile, "wb") as fh:
                            fh.write(r.read())
                            UPDATED = True
